Object subclass: #AbstractEngine
	instanceVariableNames: 'loadTime turnTime rows cols turns viewRadiusSquared attackRadiusSquared spawnRadiusSquared playerSeed currentTurn map'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!
!AbstractEngine commentStamp: 'mvdg 11/7/2011 22:05' prior: 0!
I am the base class for the External and Internal Ants engines.!


!AbstractEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 22:07'!
moveAntEast: ant
	| row col |
	row := ant row.
	col := ant col.
	self sendMoveAntEast: ant.
	map removeAntAtRow: row col: col.
	map putMyAnt: self atRow: row col: col + 1.
! !

!AbstractEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 22:07'!
moveAntNorth: ant
	| row col |
	row := ant row.
	col := ant col.
	self sendMoveAntNorth: ant.
	map removeAntAtRow: row col: col.
	map putMyAnt: ant atRow: row-1 col: col.
! !

!AbstractEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 22:07'!
moveAntSouth: ant
	| row col |
	row := ant row.
	col := ant col.
	self sendMoveAntSouth: ant.
	map removeAntAtRow: row col: col.
	map putMyAnt: self atRow: row+1 col: col.
! !

!AbstractEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 22:07'!
moveAntWest: ant
	| row col |
	row := ant row.
	col := ant col.
	self sendMoveAntWest: ant.
	map removeAntAtRow: row col: col.
	map putMyAnt: self atRow: row col: col-1.
! !


Object subclass: #AbstractMap
	instanceVariableNames: 'array height width'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!
!AbstractMap commentStamp: 'mvdg 11/7/2011 22:05' prior: 0!
I am the base class for a Grid. A Grid is just a 2 dimensional array; I could store a map, or I could store a density grid, and so forth. Use me if you want to store 2 dimensional information.!


!AbstractMap methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 21:55'!
row: aRow col: aCol
	| row col |
	row := aRow \\ height.
	col := aCol \\ width.
	^ array at: (row*width + col + 1).! !

!AbstractMap methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 21:56'!
row: aRow col: aCol put: o
	| row col |
	row := aRow \\ height.
	col := aCol \\ width.
	array at: (row * width + col + 1) put: o.! !

!AbstractMap methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 21:20'!
row: row column: column
	^ self row: row col: column.! !

!AbstractMap methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 21:20'!
row: row column: column put: o
	^ self row: row col: column put: o! !

!AbstractMap methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 21:18'!
rows: rows cols: cols
	height := rows.
	width := cols.
	array := Array new: (rows * cols).! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

AbstractMap class
	instanceVariableNames: ''!

!AbstractMap class methodsFor: 'as yet unclassified' stamp: 'mvdg 11/9/2011 22:57'!
rows: rows cols: cols
	^ self new rows: rows cols: cols.! !


AbstractEngine subclass: #ExternalEngine
	instanceVariableNames: 'inStr finished gameParameterActions turnActions bot outStr'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!
!ExternalEngine commentStamp: 'mvdg 11/7/2011 22:04' prior: 0!
I am the interface to the Google AI challenge; I write and read from the console and update an Ants game.!


!ExternalEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 16:02'!
bot: aBot
	bot := aBot.! !

!ExternalEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 22:17'!
doOneTurn
	| input key row col owner continue |
	[	input := self readSubstrings.
		key := (input at: 1) asSymbol.
		(input size > 1) ifTrue: [row := (input at: 2) asInteger ].
		(input size > 2) ifTrue: [col := (input at: 3) asInteger ].
		(input size > 3) ifTrue: [owner := (input at: 4) asInteger ].
		
		continue := (turnActions at: key ifAbsent: [[ :v :v2 :v3 | self error: 'Invalid game command'. false]]) value: row value: col value: owner.
		continue.
	] whileFalse.
	bot doTurn: currentTurn.
	self sendFinishTurn.
	self halt.! !

!ExternalEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 22:13'!
initialize
	| g |
	inStr := StdinStream new.
	outStr := FileStream stdout.
	finished := false.
	
	gameParameterActions := Dictionary new. " should really be a class var. "
	g := gameParameterActions. " shorthand. "
	" These actions are used in readGameParameters, and return true if finished. "
	g at: #turn put: [ :value | " Discard. " false. ].
	g at: #loadtime put: [ :value | loadTime := value. false. ].
	g at: #turntime put: [ :value | turnTime := value. false. ].
	g at: #rows put: [ :value | rows := value. false. ].
	g at: #cols put: [ :value | cols := value. false. ].
	g at: #turns put: [ :value | turns := value. false. ].
	g at: #viewradius2 put: [ :value | viewRadiusSquared := value. false. ].
	g at: #attackradius2 put: [ :value | attackRadiusSquared := value. false. ].
	g at: #spawnradius2 put: [ :value | spawnRadiusSquared := value. false. ].
	g at: #'player_seed' put: [ :value | playerSeed := value. false. ].
	g at: #ready put: [ :value | true ].
	
	turnActions := Dictionary new.
	g := turnActions.
	g at: #turn put: [ :row :col :owner | false. ].
	g at: #w put: [ :row :col :owner | map waterAtRow: row col: col. false ].
	g at: #f put: [ :row :col :owner | map foodAtRow: row col: col. false ].
	g at: #h put: [ :row :col :owner | 
		(owner = 0) 
			ifTrue: [ map myHillAtRow: row col: col ]
			ifFalse: [ map enemyHillAtRow: row col: col owner: owner ]. false ].
	g at: #a put: [ :row :col :owner | 
		(owner = 0) 
			ifTrue: [ 
				bot myAntAtRow: row col: col. 
				map myAntAtRow: row col: col ]
			ifFalse: [ map enemyAntAtRow: row col: col owner: owner ]. false].
	g at: #d put: [ :row :col :owner | 
		(owner = 0) 
			ifTrue: [ map myAntDiedAtRow: row col: col ]
			ifFalse: [ map enemyAntDiedAtRow: row col: col owner: owner ]. false ].
	g at: #end put: [ :row :col :owner | finished := true ].
	g at: #go put: [ :row :col :owner | true ].
	g at: #halt put: [ :row : col :owner | self halt. false. ].! !

!ExternalEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 14:34'!
initializeBot
	bot initializeWithEngine: self map: map.! !

!ExternalEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 16:19'!
initializeMap
	map := Map rows: rows cols: cols.
	
	(MapMorph new map: map) openInWindow.! !

!ExternalEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 20:55'!
readGameParameters
	| input key value |
	[	input := self readSubstrings.
		
		key := (input at: 1) asSymbol.
		(input size > 1) ifTrue: [value := (input at: 2) asInteger ].
		
		(gameParameterActions at: key ifAbsent: [[ :v | self error: 'Invalid game parameter. '. false]]) value: value.
	] whileFalse.! !

!ExternalEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/9/2011 22:17'!
readSubstrings
	| nl |
	
	[	inStr closed ifTrue: [ ^ nil ].	
		nl := inStr nextLine.
		nl isNil or: [nl isEmpty] ] whileTrue.
	
	^ nl substrings.! !

!ExternalEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 16:24'!
sendFinishTurn
	outStr nextPutAll: 'go'; cr; lf; flush.! !

!ExternalEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 16:24'!
sendMoveAntEast: ant
	self sendMoveAntSomewhere: ant.
	outStr nextPut: $E; cr; lf.! !

!ExternalEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 16:25'!
sendMoveAntNorth: ant
	self sendMoveAntSomewhere: ant.
	outStr nextPut: $N; cr; lf.! !

!ExternalEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 15:58'!
sendMoveAntSomewhere: ant
	outStr nextPutAll: 'o '.
	outStr nextPutAll: (ant row asString).
	outStr nextPut: $ .
	outStr nextPutAll: (ant col asString).
	outStr nextPut: $ .! !

!ExternalEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 16:25'!
sendMoveAntSouth: ant
	self sendMoveAntSomewhere: ant.
	outStr nextPut: $S; cr; lf.! !

!ExternalEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 16:25'!
sendMoveAntWest: ant
	self sendMoveAntSomewhere: ant.
	outStr nextPut: $W; cr; lf.! !

!ExternalEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 15:57'!
start
	self readGameParameters.
	self initializeMap.
	self initializeBot.
	self sendFinishTurn.
	[ finished ] whileFalse: [ self doOneTurn ].! !

!ExternalEngine methodsFor: 'as yet unclassified' stamp: 'mvdg 11/9/2011 22:13'!
stop
	finished := true.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ExternalEngine class
	instanceVariableNames: 'instance'!

!ExternalEngine class methodsFor: 'as yet unclassified' stamp: 'mvdg 11/9/2011 22:15'!
shutDown
	instance ifNotNil: [ instance stop ].! !

!ExternalEngine class methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 14:35'!
startUp
	instance isNil ifFalse: [ instance stop ].
	instance := self new.
	instance bot: (MyBot new).
	[ instance start ] fork.! !


Object subclass: #GridResident
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!
!GridResident commentStamp: 'mvdg 11/7/2011 22:06' prior: 0!
I am the base class of anything that could live on a map full of ants. See my subclasses.!


!GridResident methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 21:24'!
renderOn: aCanvas from: start to: end
	" Draw myself. "! !


GridResident subclass: #EnemyAnt
	instanceVariableNames: 'owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!

!EnemyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/16/2011 21:35'!
owner: anOwner
	owner := anOwner.! !

!EnemyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 22:14'!
renderOn: aCanvas from: p1 to: p2
	aCanvas fillOval: (p1 corner: p2) color: Color yellow.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

EnemyAnt class
	instanceVariableNames: ''!

!EnemyAnt class methodsFor: 'as yet unclassified' stamp: 'mvdg 11/16/2011 21:36'!
owner: anOwner
	^ self new owner: anOwner.! !


GridResident subclass: #EnemyHill
	instanceVariableNames: 'owner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!

!EnemyHill methodsFor: 'as yet unclassified' stamp: 'mvdg 11/16/2011 21:37'!
owner: anOwner
	owner := anOwner.! !

!EnemyHill methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 21:47'!
renderOn: aCanvas from: p1 to: p2
	aCanvas drawString: 'E' in: (p1 corner: p2).! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

EnemyHill class
	instanceVariableNames: ''!

!EnemyHill class methodsFor: 'as yet unclassified' stamp: 'mvdg 11/16/2011 21:37'!
owner: anOwner
	^ self new owner: anOwner.! !


AbstractEngine subclass: #LocalEngine
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!
!LocalEngine commentStamp: 'mvdg 11/7/2011 22:03' prior: 0!
I am intended to be a local, Smalltalk-only implementation of the Ants game.!



AbstractMap subclass: #Map
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!

!Map methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 22:28'!
clearFoodAroundRow: row col: col
	| r g |
	g := Ground instance.
	r := self row: row-1 col: col.
	(r isMemberOf: Food) ifTrue: [ self row: row-1 col: col put: g ].
	r := self row: row+1 col: col.
	(r isMemberOf: Food) ifTrue: [ self row: row+1 col: col put: g ].	
	r := self row: row col: col-1.
	(r isMemberOf: Food) ifTrue: [ self row: row col: col-1 put: g ].	
	r := self row: row-1 col: col+1.
	(r isMemberOf: Food) ifTrue: [ self row: row col: col+1 put: g ].! !

!Map methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 22:10'!
drawOn: aCanvas at: location
	| size |
	width ifNil: [ ^ self ].
	size := 12.
	aCanvas fillRectangle: ( location origin corner: (location origin+ ((width*size)@(height*size)))) color: (Color brown lighter lighter).
	1 to: width do: [ :x |
		1 to: height do: [ :y | | o |
			o := self row: y column: x.
			o ifNotNil: [ o renderOn: aCanvas from: (location origin + ((x*size)@(y*size))) to: (location origin + (((x+1)*size)@((y+1)*size)))]]].! !

!Map methodsFor: 'as yet unclassified' stamp: 'mvdg 11/16/2011 21:36'!
enemyAntAtRow: row col: col owner: owner
	self row: row col: col put: (EnemyAnt owner: owner).! !

!Map methodsFor: 'as yet unclassified' stamp: 'mvdg 11/16/2011 21:36'!
enemyAntDiedAtRow: row col: col owner: owner
	" Noop "! !

!Map methodsFor: 'as yet unclassified' stamp: 'mvdg 11/16/2011 21:38'!
enemyHillAtRow: row col: col owner: owner
	self row: row col: col put: (EnemyHill owner: owner).! !

!Map methodsFor: 'as yet unclassified' stamp: 'mvdg 11/16/2011 21:38'!
foodAtRow: row col: col
	self row: row col: col put: (Food instance).! !

!Map methodsFor: 'as yet unclassified' stamp: 'mvdg 11/16/2011 21:34'!
inspect
	MapMorph openOn: self.! !

!Map methodsFor: 'as yet unclassified' stamp: 'mvdg 11/16/2011 21:39'!
myAntAtRow: row col: col
	self assert: ( (self row: row col: col) isMemberOf: MyAnt ).! !

!Map methodsFor: 'as yet unclassified' stamp: 'mvdg 11/16/2011 21:40'!
myAntDiedAtRow: row col: col
	" Noop "! !

!Map methodsFor: 'as yet unclassified' stamp: 'mvdg 11/16/2011 21:40'!
myHillAtRow: row col: col
	self row: row col: col put: (MyHill new).! !

!Map methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 22:26'!
putMyAnt: ant atRow: row col: col
	| currentResident |
	currentResident := self row: row col: col.
	(ant canBeOnA: currentResident)
		ifTrue: [ 
			self row: row col: col put: ant.
			ant isNowAtRow: row col: col.
			self clearFoodAroundRow: row col: col ]
		ifFalse: [ self error: 'There''s already something there. ' ].! !

!Map methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 15:23'!
removeAntAtRow: row col: col
	" Called from MyAnt to register movement. "
	" TODO: it could be a hive... "
	" TODO: this code smells. I should implement ant movement perhaps? "
	self row: row col: col put: (Ground instance).! !

!Map methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 21:46'!
renderOn: aCanvas
	| size |
	width ifNil: [ ^ self ].
	size := 12.
	aCanvas fillRectangle: ( (0@0) corner: (width*size)@(height*size) ) color: (Color brown lighter lighter).
	1 to: width do: [ :x |
		1 to: height do: [ :y | | o |
			o := self row: y column: x.
			o ifNotNil: [ o renderOn: aCanvas from: (x*size)@(y*size) to: ((x+1)*size)@((y+1)@size)]]].! !

!Map methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 21:37'!
rows: r cols: c
	| result |
	result := super rows: r cols: c.
	array atAllPut: (Unexplored instance).
	^ result.! !

!Map methodsFor: 'as yet unclassified' stamp: 'mvdg 11/16/2011 21:41'!
waterAtRow: row col: col
	self row: row col: col put: (Water instance).! !


Morph subclass: #MapMorph
	instanceVariableNames: 'map'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!

!MapMorph methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 21:52'!
drawOn: aCanvas
	map ifNotNil: [ map drawOn: aCanvas at: bounds].! !

!MapMorph methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 21:31'!
map: aMap
	map := aMap.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

MapMorph class
	instanceVariableNames: ''!

!MapMorph class methodsFor: 'as yet unclassified' stamp: 'mvdg 11/16/2011 21:31'!
openOn: aMap
	self new map: aMap; openInWindow.! !


GridResident subclass: #MyAnt
	instanceVariableNames: 'accountedFor map row col engine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 14:48'!
accountedFor
	" Whether the engine has reported my presence in this turn. "
	^ accountedFor.! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 22:25'!
canBeOnA: aGridResident
	^ ((aGridResident isMemberOf: EnemyHill)
		or: (aGridResident isMemberOf: Ground)
		or: (aGridResident isMemberOf: MyHill)
		or: (aGridResident isMemberOf: Unexplored)).
		! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 15:18'!
canGoEast
	| currentResident |
	currentResident := map row: row col: col+1.
	^ self canBeOnA: currentResident.! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 15:17'!
canGoNorth
	| currentResident |
	currentResident := map row: (row-1) col: col.
	^ self canBeOnA: currentResident.! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 15:17'!
canGoSouth
	| currentResident |
	currentResident := map row: (row+1) col: col.
	^ self canBeOnA: currentResident.! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 15:18'!
canGoWest
	| currentResident |
	currentResident := map row: row col: col-1.
	^ self canBeOnA: currentResident.! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 15:51'!
col
	^ col.! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 22:06'!
goEast
	engine moveAntEast: self.! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 22:06'!
goNorth
	engine moveAntNorth: self.! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 22:06'!
goSouth
	engine moveAntSouth: self.! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 22:06'!
goWest
	engine moveAntWest: self.! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 14:49'!
initTurn
	accountedFor := false.! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 15:42'!
initializeWithMap: aMap engine: anEngine
	map := aMap.
	engine := anEngine.! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 15:10'!
isNowAtRow: aRow col: aCol
	row := aRow.
	col := aCol.! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 14:48'!
registerAccountedFor
	" Confirm that the engine has registered my presence. "
	accountedFor := true.! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 21:47'!
renderOn: aCanvas from: p1 to: p2
	aCanvas fillOval: (p1 corner: p2) color: Color black! !

!MyAnt methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 15:51'!
row
	^ row.! !


Object subclass: #MyBot
	instanceVariableNames: 'ants engine map'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!

!MyBot methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 16:51'!
doTurn: n
	" Put your AI implementation here. "
	ants do: [ :each | 
		(each canGoNorth) ifTrue: [ each goNorth ] ].! !

!MyBot methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 14:49'!
initTurn: n
	" Do any initialization that is needed between turns. "
	ants valuesDo: [ :each | each initTurn ].! !

!MyBot methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 15:09'!
initialize
	ants := Set new.! !

!MyBot methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 14:36'!
initializeWithEngine: anEngine map: aMap
	engine := anEngine.
	map := aMap.! !

!MyBot methodsFor: 'as yet unclassified' stamp: 'mvdg 11/20/2011 16:33'!
myAntAtRow: row col: col
	" Called by the engine when it gets information about one of our ants. "
	| currentResident  newAnt |
	currentResident := map row: row col: col.
	(ants includes: currentResident) ifFalse: [
		newAnt := MyAnt new.
		newAnt initializeWithMap: map engine: engine.
		map putMyAnt: newAnt atRow: row col: col.
		ants add: newAnt ]
	! !


GridResident subclass: #MyHill
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!

!MyHill methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 22:13'!
renderOn: aCanvas from: p1 to: p2
	aCanvas drawString: 'M' in: (p1 corner: p2).! !


GridResident subclass: #SingletonGridResident
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SingletonGridResident class
	instanceVariableNames: 'singleton'!

!SingletonGridResident class methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 22:02'!
instance
	singleton isNil ifTrue: [ singleton := self new].
	^ singleton.! !


SingletonGridResident subclass: #Food
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!
!Food commentStamp: 'mvdg 11/7/2011 22:06' prior: 0!
Singleton; I represent known food on the map.!


!Food methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 21:47'!
renderOn: aCanvas from: p1 to: p2
	aCanvas fillOval: (p1 corner: p2) color: Color brown.! !


SingletonGridResident subclass: #Ground
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!
!Ground commentStamp: 'mvdg 11/7/2011 22:06' prior: 0!
Singleton; I represent all known ground.!



Object subclass: #StdinStream
	instanceVariableNames: 'inStr delay'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!
!StdinStream commentStamp: 'mvdg 11/9/2011 21:53' prior: 0!
I'm FileStream>>stdin, except that I block until input is available.!


!StdinStream methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 20:59'!
clear
	" Read all input, discarding it. "
	[ inStr next isNil ] whileFalse.! !

!StdinStream methodsFor: 'as yet unclassified' stamp: 'mvdg 11/9/2011 22:42'!
closed
	^ inStr closed.! !

!StdinStream methodsFor: 'as yet unclassified' stamp: 'mvdg 11/9/2011 22:46'!
initialize
	inStr := FileStream stdin.
	inStr disableReadBuffering.
	delay := Delay forMilliseconds: 50.! !

!StdinStream methodsFor: 'as yet unclassified' stamp: 'mvdg 11/9/2011 22:00'!
next
	" Get the next character. Blocks. Returns nil if the stream closed while waiting. "
	| c |
	[	inStr closed ifTrue: [ ^ nil ].
		c := inStr next.
		c isNil 
	] whileTrue: [
		delay wait ].
	
	^ c.! !

!StdinStream methodsFor: 'as yet unclassified' stamp: 'mvdg 11/9/2011 22:53'!
nextLine
	| str c |
	str := WriteStream on: (String new: 60).
	
	[	c := self next.
		(c = Character cr) or: [ c = Character lf ] or: [c isNil]
	] whileFalse: [
		str nextPut: c ].
	
	^ str contents.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

StdinStream class
	instanceVariableNames: ''!

!StdinStream class methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 21:00'!
clear
	self new clear.! !


SingletonGridResident subclass: #Unexplored
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!
!Unexplored commentStamp: 'mvdg 11/7/2011 22:07' prior: 0!
Singleton; I represent all unexpored map.!



SingletonGridResident subclass: #Water
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MyBot'!

!Water methodsFor: 'as yet unclassified' stamp: 'mvdg 11/15/2011 21:47'!
renderOn: aCanvas from: p1 to: p2
	aCanvas fillRectangle: (p1 corner: p2) color: (Color blue).! !
